@@ -33,13 +33,13 @@ static int  hide_right;
 
 static FileEntryList *file_entry_list;
 static unsigned      file_count;
-static unsigned      max_userview_len;
-static unsigned      max_filename_len;
-static unsigned      max_linkname_len;
-static unsigned      global_max_filename_len;
-static unsigned      global_max_linkname_len;
+static unsigned      max_visual_userview_len;
+static unsigned      max_visual_filename_len;
+static unsigned      max_visual_linkname_len;
+static unsigned      global_max_visual_filename_len;
+static unsigned      global_max_visual_linkname_len;
 
-static void ReadFileList(DirEntry *dir_entry);
+static void ReadFileList(BOOL tagged_only, DirEntry *dir_entry);
 static void SortFileEntryList(void);
 static int  SortByName(FileEntryList *e1, FileEntryList *e2);
 static int  SortByChgTime(FileEntryList *e1, FileEntryList *e2);
@@ -50,7 +50,7 @@ static int  SortByOwner(FileEntryList *e1, FileEntryList *e2);
 static int  SortByGroup(FileEntryList *e1, FileEntryList *e2);
 static int  SortByExtension(FileEntryList *e1, FileEntryList *e2);
 static void DisplayFiles(DirEntry *de_ptr, int start_file_no, int hilight_no, int start_x);
-static void ReadGlobalFileList(DirEntry *dir_entry);
+static void ReadGlobalFileList(BOOL tagged_only, DirEntry *dir_entry);
 static void WalkTaggedFiles(int start_file, int cursor_pos, int (*fkt) (/* ??? */), WalkingPackage *walking_package);
 static BOOL IsMatchingTaggedFiles(void);
 static void RemoveFileEntry(int entry_no);
@@ -75,31 +75,31 @@ void SetFileMode(int new_file_mode)
   file_mode = new_file_mode;
   switch( file_mode )
   {
-    case MODE_1: if( max_linkname_len)
+    case MODE_1: if( max_visual_linkname_len)
 		   max_column = window_width / 
-				(max_filename_len + max_linkname_len + 45); 
+				(max_visual_filename_len + max_visual_linkname_len + 45); 
 		 else
-		   max_column = window_width / (max_filename_len + 41); 
+		   max_column = window_width / (max_visual_filename_len + 41); 
 		 break;
-    case MODE_2: if( max_linkname_len)
+    case MODE_2: if( max_visual_linkname_len)
 		   max_column = window_width / 
-                   (max_filename_len + max_linkname_len + 41);
+                   (max_visual_filename_len + max_visual_linkname_len + 41);
 		 else
-                   max_column = window_width / (max_filename_len + 37);
+                   max_column = window_width / (max_visual_filename_len + 37);
 		 break;
-    case MODE_3: max_column = window_width / (max_filename_len + 3);  
+    case MODE_3: max_column = window_width / (max_visual_filename_len + 3);  
     		 break;
-    case MODE_4: if( max_linkname_len)
+    case MODE_4: if( max_visual_linkname_len)
 		   max_column = window_width / 
-				(max_filename_len + max_linkname_len + 44); 
+				(max_visual_filename_len + max_visual_linkname_len + 44); 
 		 else
-		   max_column = window_width / (max_filename_len + 40); 
+		   max_column = window_width / (max_visual_filename_len + 40); 
 		 break;
-    case MODE_5: max_userview_len = GetUserFileEntryLength(max_filename_len, 
-					                   max_linkname_len, 
-					                   USERVIEW);
-                 if(max_userview_len)
-		   max_column = window_width / (max_userview_len + 1);
+    case MODE_5: max_visual_userview_len = GetVisualUserFileEntryLength(max_visual_filename_len, 
+					                                max_visual_linkname_len, 
+					                                USERVIEW);
+                 if(max_visual_userview_len)
+		   max_column = window_width / (max_visual_userview_len + 1);
 		 else
 		   max_column = 0;
 		 break;
@@ -130,61 +130,18 @@ void RotateFileMode(void)
 
 
 
-
-static void ReadTaggedList(DirEntry *dir_entry)
-{
-  FileEntry *fe_ptr;
-  unsigned int name_len;
-  unsigned int linkname_len;
-
-  max_filename_len = 0;
-  max_linkname_len = 0;
-
-  for( fe_ptr = dir_entry->file; fe_ptr; fe_ptr = fe_ptr->next )    
-  {
-    if (( fe_ptr->matching ) && ( fe_ptr->tagged ))
-    {
-      file_entry_list[file_count++].file = fe_ptr;
-      name_len = strlen( fe_ptr->name );
-      if( S_ISLNK( fe_ptr->stat_struct.st_mode ) )
-      {
-	linkname_len = strlen( &fe_ptr->name[name_len+1] );
-	max_linkname_len = MAX( max_linkname_len, linkname_len );
-      }
-      max_filename_len = MAX( max_filename_len, name_len );
-    }
-  }
-}
-
-static void ReadTaggedFileList(DirEntry *dir_entry)
-{
-  DirEntry  *de_ptr;
-
-  for( de_ptr=dir_entry; de_ptr; de_ptr=de_ptr->next )
-  {
-    if( de_ptr->sub_tree ) ReadTaggedFileList( de_ptr->sub_tree );
-    ReadTaggedList( de_ptr );
-    global_max_filename_len = MAX( global_max_filename_len, max_filename_len );
-    global_max_linkname_len = MAX( global_max_linkname_len, max_linkname_len );
-  }
-  max_filename_len = global_max_filename_len;
-  max_linkname_len = global_max_linkname_len;
-}
-
-
-
 static void BuildFileEntryList(DirEntry *dir_entry){
 
   if( file_entry_list ) {
     free( file_entry_list );
     file_entry_list = NULL;
   }
+
   if( !dir_entry->global_flag )  {
-     /* ... for !ANSI-Systeme ... */
-     /*----------------------------*/
-     if( dir_entry->matching_files == 0 )    {
-        file_entry_list = NULL;
-     } else {
+    /* ... for !ANSI-Systeme ... */
+    /*---------------------------*/
+    if( dir_entry->matching_files > 0 ) {
+
         if( ( file_entry_list = (FileEntryList *) 
 	  		      calloc( dir_entry->matching_files,
 				      sizeof( FileEntryList )
@@ -193,93 +150,76 @@ static void BuildFileEntryList(DirEntry *dir_entry){
         ERROR_MSG( "Calloc Failed*ABORT" );
         exit( 1 );
         }
-     }
-     file_count = 0;
-     ReadFileList( dir_entry );
-     SortFileEntryList();
-     SetFileMode( file_mode ); /* recalc */
-  }  else if (!dir_entry->tagged_flag)  {
-     	if( statistic.disk_matching_files == 0 ) {
-           file_entry_list = NULL;
-        } else {
-           if( ( file_entry_list = (FileEntryList *) 
-	  		      calloc( statistic.disk_matching_files,
+    }
+    file_count = 0;
+    ReadFileList( dir_entry->tagged_flag, dir_entry );
+    SortFileEntryList();
+    SetFileMode( file_mode ); /* recalc */
+
+    } else {
+  
+    if( statistic.disk_matching_files > 0 ) {
+
+        if( ( file_entry_list = (FileEntryList *) 
+           calloc( (dir_entry->tagged_flag) ? statistic.disk_tagged_files : statistic.disk_matching_files,
 				      sizeof( FileEntryList )
-				    ) ) 
-                              == NULL )  {
+              ) ) == NULL )  {
            ERROR_MSG( "Calloc Failed*ABORT" );
            exit( 1 );
            }
         }
         file_count = 0;
-        global_max_filename_len = 0;
-        global_max_linkname_len = 0;
-        ReadGlobalFileList( statistic.tree );
+        global_max_visual_filename_len = 0;
+        global_max_visual_linkname_len = 0;
+        ReadGlobalFileList(  dir_entry->tagged_flag, statistic.tree );
         SortFileEntryList();
         SetFileMode( file_mode ); /* recalc */
-  } else  {
-     if( statistic.disk_matching_files == 0 ) {
-        file_entry_list = NULL;
-     } else {
-        if( ( file_entry_list = (FileEntryList *) 
-	  		      calloc( statistic.disk_tagged_files,
-				      sizeof( FileEntryList )
-				    ) ) 
-                              == NULL )  {
-            ERROR_MSG( "Calloc Failed*ABORT" );
-            exit( 1 );
-        }
-     }
-     file_count = 0;
-     global_max_filename_len = 0;
-     global_max_linkname_len = 0;
-     ReadTaggedFileList( statistic.tree );
-     SortFileEntryList();
-     SetFileMode( file_mode ); /* recalc */
-  }
+    }
 }
 
 
-static void ReadFileList(DirEntry *dir_entry)
+static void ReadFileList(BOOL tagged_only, DirEntry *dir_entry)
 {
   FileEntry *fe_ptr;
   unsigned int name_len;
-  unsigned int linkname_len;
+  unsigned int visual_name_len;
+  unsigned int visual_linkname_len;
 
-  max_filename_len = 0;
-  max_linkname_len = 0;
+  max_visual_filename_len = 0;
+  max_visual_linkname_len = 0;
 
   for( fe_ptr = dir_entry->file; fe_ptr; fe_ptr = fe_ptr->next )    
   {
-    if( fe_ptr->matching ) 
+    if( fe_ptr->matching && (!tagged_only || fe_ptr->tagged) )
     {
       file_entry_list[file_count++].file = fe_ptr;
+      visual_name_len = StrVisualLength( fe_ptr->name );
       name_len = strlen( fe_ptr->name );
       if( S_ISLNK( fe_ptr->stat_struct.st_mode ) )
       {
-	linkname_len = strlen( &fe_ptr->name[name_len+1] );
-	max_linkname_len = MAX( max_linkname_len, linkname_len );
+	visual_linkname_len = StrVisualLength( &fe_ptr->name[name_len+1] );
+	max_visual_linkname_len = MAX( max_visual_linkname_len, visual_linkname_len );
       }
-      max_filename_len = MAX( max_filename_len, name_len );
+      max_visual_filename_len = MAX( max_visual_filename_len, visual_name_len );
     }
   }
 }
 
 
 
-static void ReadGlobalFileList(DirEntry *dir_entry)
+static void ReadGlobalFileList(BOOL tagged_only, DirEntry *dir_entry)
 {
   DirEntry  *de_ptr;
 
   for( de_ptr=dir_entry; de_ptr; de_ptr=de_ptr->next )
   {
-    if( de_ptr->sub_tree ) ReadGlobalFileList( de_ptr->sub_tree );
-    ReadFileList( de_ptr );
-    global_max_filename_len = MAX( global_max_filename_len, max_filename_len );
-    global_max_linkname_len = MAX( global_max_linkname_len, max_linkname_len );
+    if( de_ptr->sub_tree ) ReadGlobalFileList( tagged_only, de_ptr->sub_tree );
+    ReadFileList( tagged_only, de_ptr );
+    global_max_visual_filename_len = MAX( global_max_visual_filename_len, max_visual_filename_len );
+    global_max_visual_linkname_len = MAX( global_max_visual_linkname_len, max_visual_linkname_len );
   }
-  max_filename_len = global_max_filename_len;
-  max_linkname_len = global_max_linkname_len;
+  max_visual_filename_len = global_max_visual_filename_len;
+  max_visual_linkname_len = global_max_visual_linkname_len;
 }
 
 
@@ -333,9 +273,9 @@ static int SortByName(FileEntryList *e1, FileEntryList *e2)
         return( - (strcmp( e1->file->name, e2->file->name ) ) );
   else
      if (order)
-        return( strcasecmp( e1->file->name, e2->file->name ) );
+        return( StrCaseCmp( e1->file->name, e2->file->name ) );
      else
-        return( - (strcasecmp( e1->file->name, e2->file->name ) ) );
+        return( - (StrCaseCmp( e1->file->name, e2->file->name ) ) );
 }
 
 
@@ -349,7 +289,7 @@ static int SortByExtension(FileEntryList *e1, FileEntryList *e2)
   ext1 = GetExtension(e1->file->name);
   ext2 = GetExtension(e2->file->name);
   cmp=strcmp( ext1, ext2 );
-  casecmp=strcasecmp( ext1, ext2 );
+  casecmp=StrCaseCmp( ext1, ext2 );
 
   if (do_case && !cmp)
       return SortByName( e1, e2 );
@@ -364,9 +304,9 @@ static int SortByExtension(FileEntryList *e1, FileEntryList *e2)
         return( - (strcmp( ext1, ext2 ) ) );
   else
      if (order)
-        return( strcasecmp( ext1, ext2 ) );
+        return( StrCaseCmp( ext1, ext2 ) );
      else
-        return( - (strcasecmp( ext1, ext2 ) ) );
+        return( - (StrCaseCmp( ext1, ext2 ) ) );
 }
 
 
@@ -436,9 +376,9 @@ static int SortByOwner(FileEntryList *e1, FileEntryList *e2)
         return( - (strcmp( o1, o2 ) ) );
   else
      if (order)
-        return( strcasecmp( o1, o2 ) );
+        return( StrCaseCmp( o1, o2 ) );
      else
-        return( - (strcasecmp( o1, o2 ) ) );
+        return( - (StrCaseCmp( o1, o2 ) ) );
 }
 
 
@@ -469,9 +409,9 @@ static int SortByGroup(FileEntryList *e1, FileEntryList *e2)
         return( - (strcmp( g1, g2 ) ) );
   else
      if (order)
-        return( strcasecmp( g1, g2 ) );
+        return( StrCaseCmp( g1, g2 ) );
      else
-        return( - (strcasecmp( g1, g2 ) ) );
+        return( - (StrCaseCmp( g1, g2 ) ) );
 }
 
 
@@ -485,22 +425,24 @@ void SetKindOfSort(int new_kind_of_sort)
 
 static void RemoveFileEntry(int entry_no)
 {
-  int i, n, l;
+  int i, n;
   FileEntry *fe_ptr;
+  int visual_name_len, name_len;
 
-  max_filename_len = 0;
-  max_linkname_len = 0;
+  max_visual_filename_len = 0;
+  max_visual_linkname_len = 0;
   n = file_count - 1;
 
   for( i=0; i < n; i++ )
   {
     if( i >= entry_no ) file_entry_list[i] = file_entry_list[i+1];    
     fe_ptr = file_entry_list[i].file;
-    l = strlen( fe_ptr->name );
-    max_filename_len = MAX( (int)max_filename_len, l );
+    visual_name_len = StrVisualLength( fe_ptr->name );
+    name_len = strlen( fe_ptr->name );
+    max_visual_filename_len = MAX( (int)max_visual_filename_len, visual_name_len );
     if( S_ISLNK( fe_ptr->stat_struct.st_mode ) )
     {
-      max_linkname_len = MAX( max_filename_len, strlen( &fe_ptr->name[l+1] ) );
+      max_visual_linkname_len = MAX( max_visual_filename_len, StrVisualLength( &fe_ptr->name[name_len+1] ) );
     }
   }
 
@@ -513,11 +455,12 @@ static void RemoveFileEntry(int entry_no)
 
 static void ChangeFileEntry(void)
 {
-  int i, n, l;
+  int i, n;
   FileEntry *fe_ptr;
+  int visual_name_len, name_len;
 
-  max_filename_len = 0;
-  max_linkname_len = 0;
+  max_visual_filename_len = 0;
+  max_visual_linkname_len = 0;
   n = file_count - 1;
 
   for( i=0; i < n; i++ )
@@ -525,11 +468,12 @@ static void ChangeFileEntry(void)
     fe_ptr = file_entry_list[i].file;
     if( fe_ptr )
     {
-      l = strlen( fe_ptr->name );
-      max_filename_len = MAX( (int)max_filename_len, l );
+      visual_name_len = StrVisualLength( fe_ptr->name );
+      name_len = strlen( fe_ptr->name );
+      max_visual_filename_len = MAX( (int)max_visual_filename_len, visual_name_len );
       if( S_ISLNK( fe_ptr->stat_struct.st_mode ) )
       {
-        max_linkname_len = MAX( max_filename_len, strlen( &fe_ptr->name[l+1] ) );
+        max_visual_linkname_len = MAX( max_visual_filename_len, StrVisualLength( &fe_ptr->name[name_len+1] ) );
       }
     }
   }
@@ -577,6 +521,8 @@ static void PrintFileEntry(int entry_no, int y, int x, unsigned char hilight, in
   int  ef_window_width;
   char *sym_link_name = NULL;
   char type_of_file = ' ';
+  int  filename_width = 0;
+  int  linkname_width = 0;
 
 
   ef_window_width = window_width - 2; /* Effektive Window-Width */
@@ -603,6 +549,16 @@ static void PrintFileEntry(int entry_no, int y, int x, unsigned char hilight, in
     sym_link_name = "";
 
 
+#ifdef WITH_UTF8
+  filename_width = max_visual_filename_len + ( strlen(fe_ptr->name) - StrVisualLength(fe_ptr->name) );
+  if( fe_ptr && S_ISLNK( fe_ptr->stat_struct.st_mode ) )
+    linkname_width = max_visual_linkname_len + ( strlen(sym_link_name) - StrVisualLength(sym_link_name) );
+#else
+  filename_width = max_visual_filename_len;
+  linkname_width = max_visual_linkname_len;
+#endif
+
+
   type_of_file = GetTypeOfFile(fe_ptr->stat_struct);
   
   switch( file_mode )
@@ -621,8 +577,8 @@ static void PrintFileEntry(int entry_no, int y, int x, unsigned char hilight, in
 		    {
 #ifdef HAS_LONGLONG
 		      (void) sprintf( format, "%%c%%c%%-%ds %%10s %%3d %%11lld %%12s -> %%-%ds", 
-				      max_filename_len,
-				      max_linkname_len
+				      filename_width,
+				      linkname_width
 				    );
 		      
 		      (void) sprintf( line_buffer, format,
@@ -637,8 +593,8 @@ static void PrintFileEntry(int entry_no, int y, int x, unsigned char hilight, in
 				    );
 #else
 		      (void) sprintf( format, "%%c%%c%%-%ds %%10s %%3d %%7d %%12s -> %%-%ds", 
-				      max_filename_len,
-				      max_linkname_len
+				      filename_width,
+				      linkname_width
 				    );
     
 		      (void) sprintf( line_buffer, format,
@@ -658,7 +614,7 @@ static void PrintFileEntry(int entry_no, int y, int x, unsigned char hilight, in
 #ifdef HAS_LONGLONG
 		      (void) sprintf( format, "%%c%%c%%%c%ds %%10s %%3d %%11lld %%12s", 
                                       justify,
-				      max_filename_len 
+				      filename_width 
 				    );
     
 		      (void) sprintf( line_buffer, format,
@@ -673,7 +629,7 @@ static void PrintFileEntry(int entry_no, int y, int x, unsigned char hilight, in
 #else
 		      (void) sprintf( format, "%%c%%c%%%c%ds %%10s %%3d %%7d %%12s", 
                                       justify,
-				      max_filename_len 
+				      filename_width 
 				    );
     
 		      (void) sprintf( line_buffer, format,
@@ -693,14 +649,14 @@ static void PrintFileEntry(int entry_no, int y, int x, unsigned char hilight, in
 		    /* Empty Entry */
 		    /*-------------*/
 
-		    (void) sprintf( format, "%%-%ds", max_filename_len + 42 );
+		    (void) sprintf( format, "%%-%ds", max_visual_filename_len + 42 );
 		    (void) sprintf( line_buffer, format, "" );
 		  }
 
-		  if( max_linkname_len ) 
-		    pos_x = x * (max_filename_len + max_linkname_len + 47); 
+		  if( max_visual_linkname_len ) 
+		    pos_x = x * (max_visual_filename_len + max_visual_linkname_len + 47); 
 		  else                   
-		    pos_x = x * (max_filename_len + 43); 
+		    pos_x = x * (max_visual_filename_len + 43); 
 		  break;
     
     case MODE_2 : if( fe_ptr )
@@ -728,8 +684,8 @@ static void PrintFileEntry(int entry_no, int y, int x, unsigned char hilight, in
 #ifdef HAS_LONGLONG
                       (void) sprintf( format, "%%c%%c%%%c%ds %%10lld %%-12s %%-12s -> %%-%ds",
                                       justify,
-			              max_filename_len,
-			              max_linkname_len
+			              filename_width,
+			              linkname_width
 				      );
 		      (void) sprintf( line_buffer, format,
 				      (fe_ptr->tagged) ? TAGGED_SYMBOL : ' ',
@@ -743,8 +699,8 @@ static void PrintFileEntry(int entry_no, int y, int x, unsigned char hilight, in
 #else
                       (void) sprintf( format, "%%c%%c%%%c%ds %%8u  %%-12s  %%-12s -> %%-%ds",
                                       justify,
-			              max_filename_len,
-			              max_linkname_len
+			              filename_width,
+			              linkname_width
 				      );
 		      (void) sprintf( line_buffer, format,
 				      (fe_ptr->tagged) ? TAGGED_SYMBOL : ' ',
@@ -762,7 +718,7 @@ static void PrintFileEntry(int entry_no, int y, int x, unsigned char hilight, in
 #ifdef HAS_LONGLONG
                       (void) sprintf( format, "%%c%%c%%%c%ds %%10lld %%-12s %%-12s",
                                       justify,
-			              max_filename_len
+			              filename_width
 				      );
 		      (void) sprintf( line_buffer, format,
 				      (fe_ptr->tagged) ? TAGGED_SYMBOL : ' ',
@@ -775,7 +731,7 @@ static void PrintFileEntry(int entry_no, int y, int x, unsigned char hilight, in
 #else
                       (void) sprintf( format, "%%c%%c%%%c%ds %%8u  %%-12s  %%-12s",
                                       justify,
-			              max_filename_len
+			              filename_width
 				      );
 		      (void) sprintf( line_buffer, format,
 				      (fe_ptr->tagged) ? TAGGED_SYMBOL : ' ',
@@ -794,21 +750,21 @@ static void PrintFileEntry(int entry_no, int y, int x, unsigned char hilight, in
 		    /* Empty-Entry */
 		    /*-------------*/
 
-		    (void) sprintf( format, "%%-%ds", max_filename_len + 38 );
+		    (void) sprintf( format, "%%-%ds", max_visual_filename_len + 38 );
 		    (void) sprintf( line_buffer, format, "" );
 		  }
 
-		  if( max_linkname_len ) 
-                    pos_x = x * (max_filename_len + max_linkname_len + 43);
+		  if( max_visual_linkname_len ) 
+                    pos_x = x * (max_visual_filename_len + max_visual_linkname_len + 43);
 		  else                   
-                    pos_x = x * (max_filename_len + 39);
+                    pos_x = x * (max_visual_filename_len + 39);
 		  break;
     
     case MODE_3 : if( fe_ptr )
 		  {
 		    (void) sprintf( format, "%%c%%c%%%c%ds", 
                                     justify, 
-                                    max_filename_len );
+                                    filename_width );
 
 		    (void) sprintf( line_buffer, format,
 				    (fe_ptr->tagged) ? TAGGED_SYMBOL : ' ',
@@ -821,11 +777,11 @@ static void PrintFileEntry(int entry_no, int y, int x, unsigned char hilight, in
 		    /* Empty-Entry */
 		    /*-------------*/
 
-		    (void) sprintf( format, "%%-%ds", max_filename_len + 2 );
+		    (void) sprintf( format, "%%-%ds", max_visual_filename_len + 2 );
 		    (void) sprintf( line_buffer, format, "" );
 		  }
 
-		  pos_x = x * (max_filename_len + 3); 
+		  pos_x = x * (max_visual_filename_len + 3); 
 		  break;
     
     case MODE_4 : if( fe_ptr )
@@ -837,8 +793,8 @@ static void PrintFileEntry(int entry_no, int y, int x, unsigned char hilight, in
 		    {
                       (void) sprintf( format, "%%c%%c%%%c%ds Chg: %%12s  Acc: %%12s -> %%-%ds",
                                       justify,
-				      max_filename_len,
-				      max_linkname_len
+				      filename_width,
+				      linkname_width
 				    );
 		      (void) sprintf( line_buffer, format,
 				      (fe_ptr->tagged) ? TAGGED_SYMBOL : ' ',
@@ -853,7 +809,7 @@ static void PrintFileEntry(int entry_no, int y, int x, unsigned char hilight, in
 		    {
                       (void) sprintf( format, "%%c%%c%%%c%ds Chg: %%12s  Acc: %%12s",
                                       justify,
-				      max_filename_len
+				      filename_width
 				    );
 		      (void) sprintf( line_buffer, format,
 				      (fe_ptr->tagged) ? TAGGED_SYMBOL : ' ',
@@ -869,20 +825,20 @@ static void PrintFileEntry(int entry_no, int y, int x, unsigned char hilight, in
 		    /* Empty-Entry */
 		    /*-------------*/
 
-		    (void) sprintf( format, "%%-%ds", max_filename_len + 39 );
+		    (void) sprintf( format, "%%-%ds", max_visual_filename_len + 39 );
 		    (void) sprintf( line_buffer, format, "" );
 		  }
 
 
-		  if( max_linkname_len ) 
-		    pos_x = x * (max_filename_len + max_linkname_len + 44); 
+		  if( max_visual_linkname_len ) 
+		    pos_x = x * (max_visual_filename_len + max_visual_linkname_len + 44); 
 		  else                   
-		    pos_x = x * (max_filename_len + 40); 
+		    pos_x = x * (max_visual_filename_len + 40); 
 		  break;
     
     case MODE_5 : if( fe_ptr )
 		  {
- 		    BuildUserFileEntry(fe_ptr,  max_filename_len, max_linkname_len, 
+ 		    BuildUserFileEntry(fe_ptr,  filename_width, linkname_width, 
 		        USERVIEW, 
 		        200, line_buffer);
 		  }
@@ -891,10 +847,10 @@ static void PrintFileEntry(int entry_no, int y, int x, unsigned char hilight, in
 		    /* Empty-Entry */
 		    /*-------------*/
 
-		    (void) sprintf( format, "%%-%ds", max_userview_len );
+		    (void) sprintf( format, "%%-%ds", max_visual_userview_len );
 		    (void) sprintf( line_buffer, format, "" );
 		  }
-		  pos_x = x * (max_userview_len + 1);
+		  pos_x = x * (max_visual_userview_len + 1);
 		  break;
 
   }
@@ -915,16 +871,20 @@ static void PrintFileEntry(int entry_no, int y, int x, unsigned char hilight, in
   }
   else
   {
+    int line_end_pos;
+
     /* ... does not fit; use start_x */
     /*-------------------------------*/
     
     if( n > ( start_x + ef_window_width ) )
-      line_ptr = &line_buffer[start_x];  /* TODO: UTF-8 */
+      line_ptr = &line_buffer[VisualPositionToBytePosition(line_buffer, start_x)];
     else
-      line_ptr = &line_buffer[n - ef_window_width];  /* TODO: UTF-8 */
+      line_ptr = &line_buffer[VisualPositionToBytePosition(line_buffer, n - ef_window_width)];
+
     hide_left = start_x;
-    hide_right = n - start_x - ef_window_width;
-    line_ptr[ef_window_width] = '\0';
+    line_end_pos = VisualPositionToBytePosition(line_ptr, ef_window_width);
+    hide_right = StrVisualLength(&line_ptr[line_end_pos]);
+    line_ptr[line_end_pos] = '\0';
   }
  
 #ifdef NO_HIGHLIGHT
@@ -947,7 +907,10 @@ static void PrintFileEntry(int entry_no, int y, int x, unsigned char hilight, in
   if( hilight ) wattrset( file_window, 0 );
 #endif /* NO_HIGHLIGHT */
  
- 
+#ifdef WITH_UTF8
+  if(x_step == 1 && start_x > 0)
+    wclrtoeol(file_window); /* maybe needed if filename contains wide characters */
+#endif
 }
 
 
@@ -1106,7 +1069,7 @@ static void fmoveright(int *start_file, int *cursor_pos, int *start_x,DirEntry *
                       TRUE ,
                       *start_x
                       );
-      if( hide_right < 0 ) (*start_x)--;
+      if( hide_right <= 0 ) (*start_x)--;
    }
    else if( *start_file + *cursor_pos >= (int)file_count - 1 )
    {
@@ -2745,7 +2708,31 @@ int HandleFileWindow(DirEntry *dir_entry)
 		      need_dsp_help = TRUE;
 		      break;
 
-      default:        beep();
+    case KEY_F(28):                  /* Ctrl-F4 */
+    case KEY_F(4+12):                /* Shift-F4 */
+        
+	/* Toggle mode (if possible) */
+        if(dir_entry->tagged_files)
+          dir_entry->tagged_flag = !dir_entry->tagged_flag;
+        else
+          dir_entry->tagged_flag = FALSE;
+
+        BuildFileEntryList ( dir_entry );
+
+        dir_entry->start_file = 0;
+        dir_entry->cursor_pos = 0;
+        DisplayFiles
+        (
+            dir_entry,
+            dir_entry->start_file,
+            dir_entry->start_file + dir_entry->cursor_pos,
+            start_x
+        );
+        maybe_change_x_step = TRUE;
+        break;
+
+     default:
+        beep();
 		      break;
     }
   } while( ch != CR && ch != ESC && ch != LOGIN_ESC );
@@ -3079,7 +3066,7 @@ static void ListJump( DirEntry * dir_entry, char *str )
     }
 
     n = strlen(str);
-    if((newStr = malloc(n+2)) == NULL) {
+    if((newStr = (char *)malloc(n+2)) == NULL) {
       ERROR_MSG( "Malloc failed*ABORT" );
       exit( 1 );
     }
@@ -3106,7 +3093,7 @@ static void ListJump( DirEntry * dir_entry, char *str )
     for( i=tmp2; i < file_count; i++ )
     {
         fe_ptr = file_entry_list[i].file;
-	if(!strncasecmp(newStr, fe_ptr->name, n+1))
+	if(!StrNCaseCmp(newStr, fe_ptr->name, n+1))
           break;
     }
 
