@@ -13,98 +13,226 @@
  ***************************************************************************/
 
 
-char *StrLeft(const char *str, size_t count)
+char *StrLeft(const char *str, size_t visible_count)
 {
-#ifdef WITH_UTF8
-  mbstate_t state;
-  char *p;
-#endif
-  char *rez, *tmp;
-  size_t len, i;
+  char *result;
+  size_t len;
+  int  left_bytes;
 
 #ifdef WITH_UTF8
-  memset(&state, 0, sizeof(state));
+  mbstate_t state;
+  const char *s, *s_start;;
+  int pos = 0;
 #endif
-  if (count == 0) return(Strdup(""));
+
+  if (visible_count == 0) 
+    return(Strdup(""));
+
   len = StrVisualLength(str);
-  if (count >= len) return(Strdup(str));
+  if (visible_count >= len) 
+    return(Strdup(str));
   
-  len = 0;
-
-  tmp = Strdup(str);
 #ifdef WITH_UTF8
-  p = tmp;
-#endif
-  for (i = 0; i < count; i++) {
-#ifdef WITH_UTF8
-    len += mbrlen(p, 4, &state);
-    p = tmp + len;
-#else
-    len++;
-#endif
-  }
-  free(tmp);
-  
-  rez = Strndup(str, len);
-  rez[len] = '\0';
-  return(rez);
-}
 
-char *StrRight(const char *str, size_t count)
-{
-#ifdef WITH_UTF8
-  mbstate_t state;
-#endif
-  char *rez, *p, *tmp;
-  size_t byte_len, char_len, tmp_len, i;
+  s_start = s = str;
 
-#ifdef WITH_UTF8
-  memset(&state, 0, sizeof(state));
-#endif
+  while(*s) {
 
-  if (count == 0) return(Strdup(""));
+    wchar_t wc;
+    size_t sz;
+    int width;
 
-  byte_len = strlen(str);
-  char_len = StrVisualLength(str);
-  
-  if (count > char_len) count = char_len;
-  
-  tmp = Strdup(str);
-  p = tmp;
-  i = 0;
-  rez = NULL;
-  while ( (p - tmp) < byte_len ) {
-    if (i == (char_len - count) ) {
-      rez = Strdup(p);
+    s_start = s;
+    memset(&state, 0, sizeof(state));
+    sz = mbrtowc(&wc, s, 4, &state);
+    if(sz == (size_t)-1 || sz == (size_t)-2) {
+      if( (*s++ & 0xc0) == 0xc0) {  /* skip to next char */
+        while( (*s & 0xc0) == 0x80) 
+          s++;
+      }
+      pos += 4; /* assume worst case */
+    } else {
+      s += sz;
+      width = wcwidth(wc);
+      if(width >= 0)
+        pos += width;
+      else
+        pos++;
     }
-#ifdef WITH_UTF8    
-    tmp_len = mbrlen(p, 4, &state);
-#else
-    tmp_len = 1;
-#endif
-    p += tmp_len;
-    i++;
+
+    if(pos > visible_count)
+      break;  /* exceeds limit */
   }
 
-  free(tmp);
-  return(rez);
+  if(*s)
+    left_bytes = s_start - str; 
+  else
+    left_bytes = s - str; 
+
+#else
+  left_bytes = visible_count; 
+#endif
+
+  result = Strndup(str, left_bytes);
+  return(result);
 }
 
+
+char *StrRight(const char *str, size_t visible_count)
+{
+  char *result;
+  size_t visual_len;
+
+#ifdef WITH_UTF8
+  int left_bytes, pos = 0;
+  mbstate_t state;
+  const char *s, *s_start;
+#endif
+
+  if (visible_count == 0) 
+    return(Strdup(""));
+
+  visual_len = StrVisualLength(str);
+  if(visual_len <= visible_count)
+    return(Strdup(str));
+  
+#ifdef WITH_UTF8
+
+  s_start = s = str;
+
+  while(*s) {
+
+    wchar_t wc;
+    size_t sz;
+    int width;
+
+    s_start = s;
+    memset(&state, 0, sizeof(state));
+    sz = mbrtowc(&wc, s, 4, &state);
+    if(sz == (size_t)-1 || sz == (size_t)-2) {
+      if( (*s++ & 0xc0) == 0xc0) {  /* skip to next char */
+        while( (*s & 0xc0) == 0x80) 
+          s++;
+      }
+      pos += 4; /* assume worst case */
+    } else {
+      s += sz;
+      width = wcwidth(wc);
+      if(width >= 0)
+        pos += width;
+      else
+        pos++;
+    }
+
+    if((visual_len - pos) < visible_count)
+      break;  /* fits */
+  }
+  
+  if(*s)
+    left_bytes = s_start - str; 
+  else
+    left_bytes = s - str; 
+
+  result = Strdup(&str[left_bytes]);
+
+#else
+  result = Strdup( &str[visual_len - visible_count] );
+#endif
+
+  return(result);
+}
+
+
+
 int StrVisualLength(const char *str)
 {
+  int len;
+
 #ifdef WITH_UTF8
+
+  int pos = 0;
+  size_t sz;
   mbstate_t state;
-  int len = 0;
-  
-  memset(&state, '\0', sizeof(state));
-  len = mbsrtowcs(NULL, &str, strlen(str), &state);
-  if(len < 0) {
-    /* Invalid multibyte sequence */
-    len = strlen(str);
-  }
+  const char *s = str;
+  wchar_t buffer[PATH_LENGTH+1];
+
+  do {
+    memset(&state, 0, sizeof(state));
+    sz = mbrtowc(&buffer[pos], s, 4, &state);
+    if( sz == (size_t) -1 || sz == (size_t)-2 ) {
+      if( (*s++ & 0xc0) == 0xc0) {  /* skip to next char */
+        while( (*s & 0xc0) == 0x80) 
+          s++;
+      }
+      buffer[pos++]   = L'#';  /* Assume worst case: */
+      buffer[pos++]   = L'#';  /* 1 wide char produces 4 visible chars */
+      buffer[pos++]   = L'#';
+      buffer[pos]     = L'#';
+    } else {
+      s += sz;
+    }
+    pos++;
+  } while(sz != 0);
+
+  len = wcswidth(buffer, PATH_LENGTH);
+
+  if(len < 0)
+    len = pos; /* should not happen */
+
 #else
-  return(strlen(str));
+  len = strlen(str);
 #endif
+
+  return len;
+}
+
+
+/* returns byte position for visual position */
+int VisualPositionToBytePosition(const char *str, int visual_pos)
+{
+
+#ifdef WITH_UTF8
+
+  mbstate_t state;
+  const char *s, *s_start;
+  int pos = 0;
+
+  s_start = s = str;
+
+  while(*s) {
+
+    wchar_t wc;
+    size_t sz;
+    int width;
+
+    s_start = s;
+    memset(&state, 0, sizeof(state));
+    sz = mbrtowc(&wc, s, 4, &state);
+    if(sz == (size_t)-1 || sz == (size_t)-2) {
+      if( (*s++ & 0xc0) == 0xc0) {  /* skip to next char */
+        while( (*s & 0xc0) == 0x80) 
+          s++;
+      }
+      pos += 4; /* assume worst case */
+    } else {
+      s += sz;
+      width = wcwidth(wc);
+      if(width > 0)
+	pos += width;
+      else
+	pos++;
+    }
+
+    if(pos > visual_pos)
+      return( s_start - str );
+  }
+
+  return( s - str );
+ 
+#else
+  return visual_pos;
+#endif
+
 }
 
 
@@ -116,7 +244,7 @@ int InputString(char *s, int y, int x, int cursor_pos, int length, char *term)
 {
   int p;                       /* Aktuelle Position                   */
   int c1;                      /* Gelesenes Zeichen                   */
-  int i;                       /* Laufvariable                        */
+  int i, n;                    /* Laufvariable                        */
   char *pp;
   BOOL len_flag = FALSE;
   char path[PATH_LENGTH + 1];
@@ -192,30 +320,34 @@ int InputString(char *s, int y, int x, int cursor_pos, int length, char *term)
                               break;
         case KEY_END        : p = StrVisualLength( s );
                               break;
-        case KEY_DC         : if( p < StrVisualLength(s) )
+        case KEY_DC         : n = StrVisualLength(s);
+                              if( p < n )
                               {
 			        ls = StrLeft(s, p);
-				rs = StrRight(s, StrVisualLength(s) - p - 1);
+				rs = StrRight(s, n - p - 1);
 				strcpy(s, ls);
 				strcat(s, rs);
 				free(ls);
 				free(rs);
 				MvAddStr( y, x, s );
-                                addch( '_' );
+                                for(i=StrVisualLength(s); i < n; i++)
+                                  addch( '_' );
                               }
                               break;
 	case 0x08           :
         case 0x7F           :
         case KEY_BACKSPACE  : if( p > 0 )
                               {
+                                n = StrVisualLength(s);
 			        ls = StrLeft(s, p - 1);
-				rs = StrRight(s, StrVisualLength(s) - p);
+				rs = StrRight(s, n - p);
 				strcpy(s, ls);
 				strcat(s, rs);
 				free(ls);
 				free(rs);
 				MvAddStr( y, x, s );
-                                addch( '_' );
+                                for(i=StrVisualLength(s); i < n; i++)
+                                  addch( '_' );
                                 p--;
                               }
                               else
