@@ -504,18 +504,36 @@ char *FormFilename(char *dest, char *src, unsigned int max_len)
   int begin;
   unsigned int l;
 
-  l = strlen(src);
+  l = strlen(src); /* TODO: UTF-8 */
   begin = 0;
 
   if( l <= max_len )
     return( strcpy( dest, src ) );
   else
   {
-    for(i=0; i < (int) max_len - 4; i++)
-      if( src[l - i] == FILE_SEPARATOR_CHAR || src[l - i] == '\\' )
-        begin = l - i;
     (void) strcpy( dest, "/..." );
-    return( strcat(dest, &src[begin] ) );
+
+    if(max_len < 4) {
+      dest[max_len] = '\0';
+    } else {
+      for(i=0; i < (int) max_len - 4; i++)
+        if( src[l - i] == FILE_SEPARATOR_CHAR || src[l - i] == '\\' )
+          begin = l - i;
+      
+      if(begin > 0) {
+        strcat(dest, &src[begin] );
+        if(strlen(dest) > max_len)
+          strcpy( &dest[max_len - 3], "..." );
+      } else {
+        for(i=0; i < l; i++)
+	  if( src[i] == FILE_SEPARATOR_CHAR || src[i] == '\\' )
+	    begin = i;  /* find last '/' */
+        strcat(dest, &src[begin] );
+        if(strlen(dest) > max_len)
+          strcpy( &dest[max_len - 3], "..." );
+      }
+    }
+    return dest;
   }
 }
 
@@ -842,6 +860,43 @@ int Strrcmp(char *s1, char* s2)/*compares in reverse order 2 strings*/
 
 
 
+int StrNCaseCmp(char *s1, char *s2, unsigned int n)
+{
+  int c1, c2;
+  unsigned char *us1 = (unsigned char*)s1;
+  unsigned char *us2 = (unsigned char*)s2;
+
+  if(n == 0)
+    return(0);
+
+  do
+  {
+    c1 = toupper(*us1++);
+    c2 = toupper(*us2++);
+  } while((c1 == c2) && (c1 != '\0') && (--n != 0));
+
+  return(c1 - c2);
+}
+
+
+
+int StrCaseCmp(char *s1, char *s2)
+{
+  int c1, c2;
+  unsigned char *us1 = (unsigned char*)s1;
+  unsigned char *us2 = (unsigned char*)s2;
+
+  do
+  {
+    c1 = toupper(*us1++);
+    c2 = toupper(*us2++);
+  } while((c1 == c2) && (c1 != '\0'));
+
+  return(c1 - c2);
+}
+
+
+
 /* NeXT does not define strdup */
 char *Strdup(const char *s)
 {
@@ -883,7 +938,7 @@ char *GetExtension(char *filename)
   if(cptr == NULL) return "";
 
   if(cptr == filename) return "";
-  // filenames beginning with a dot are not an extension
+  /* filenames beginning with a dot are not an extension */
 
   return(cptr + 1);
 }
@@ -908,7 +963,7 @@ void StrCp(char *dest, const char *src)
 
 
 int BuildUserFileEntry(FileEntry *fe_ptr, 
-			int max_filename_len, int max_linkname_len, 
+			int filename_width, int linkname_width, 
 			char *template, int linelen, char *line)
 {
   char attributes[11];
@@ -956,8 +1011,8 @@ int BuildUserFileEntry(FileEntry *fe_ptr,
   }
 
 
-  sprintf(format1, "%%-%ds", max_filename_len);
-  sprintf(format2, "%%-%ds", max_linkname_len);
+  sprintf(format1, "%%-%ds", filename_width);
+  sprintf(format2, "%%-%ds", linkname_width);
 
   for(sptr=template, dptr=buffer; *sptr; ) {
 
@@ -973,7 +1028,7 @@ int BuildUserFileEntry(FileEntry *fe_ptr,
         n = sprintf(dptr, "%3d", (int)fe_ptr->stat_struct.st_nlink);
       } else if(!strncmp(sptr, FILESIZE_VIEWNAME, 3)) {
 #ifdef HAS_LONGLONG
-        n = sprintf(dptr, "%7lld", (LONGLONG) fe_ptr->stat_struct.st_size);
+        n = sprintf(dptr, "%11lld", (LONGLONG) fe_ptr->stat_struct.st_size);
 #else
         n = sprintf(dptr, "%7d", fe_ptr->stat_struct.st_size);
 #endif
@@ -987,7 +1042,7 @@ int BuildUserFileEntry(FileEntry *fe_ptr,
         n = sprintf(dptr, "%-8s", group_name_ptr);
       } else if(!strncmp(sptr, INODE_VIEWNAME, 3)) {
 #ifdef HAS_LONGLONG
-        n = sprintf(dptr, "%7lld", (LONGLONG)fe_ptr->stat_struct.st_ino);
+        n = sprintf(dptr, "%11lld", (LONGLONG)fe_ptr->stat_struct.st_ino);
 #else
         n = sprintf(dptr, "%7ld", (int)fe_ptr->stat_struct.st_ino);
 #endif
@@ -1018,7 +1073,7 @@ int BuildUserFileEntry(FileEntry *fe_ptr,
   
 
 
-int GetUserFileEntryLength( int max_filename_len, int max_linkname_len, char *template)
+int GetVisualUserFileEntryLength( int max_visual_filename_len, int max_visual_linkname_len, char *template)
 {
   int  len, n;
   char *sptr;
@@ -1031,7 +1086,7 @@ int GetUserFileEntryLength( int max_filename_len, int max_linkname_len, char *te
       if(!strncmp(sptr, TAGSYMBOL_VIEWNAME, 3)) {
         n=1;
       } else if(!strncmp(sptr, FILENAME_VIEWNAME, 3)) {
-        n = max_filename_len;
+        n = max_visual_filename_len;
       } else if(!strncmp(sptr, ATTRIBUTE_VIEWNAME, 3)) {
         n = 10;
       } else if(!strncmp(sptr, LINKCOUNT_VIEWNAME, 3)) {
@@ -1041,7 +1096,7 @@ int GetUserFileEntryLength( int max_filename_len, int max_linkname_len, char *te
       } else if(!strncmp(sptr, MODTIME_VIEWNAME, 3)) {
         n = 12;
       } else if(!strncmp(sptr, SYMLINK_VIEWNAME, 3)) {
-        n = max_linkname_len;
+        n = max_visual_linkname_len;
       } else if(!strncmp(sptr, UID_VIEWNAME, 3)) {
         n = 8;
       } else if(!strncmp(sptr, GID_VIEWNAME, 3)) {
