@@ -12,7 +12,7 @@
 #include <ctype.h>
 #include <math.h>
 
-#if defined(__OpenBSD__) || defined(__NetBSD__) || defined(__FreeBSD__) || defined(__NetBSD__) || defined(__APPLE__)
+#if defined(__OpenBSD__) || defined(__NetBSD__) || defined(__FreeBSD__) || defined(__NetBSD__) || defined(__APPLE__) || defined(SVR4)
 #include <locale.h>
 #endif
 
@@ -1081,10 +1081,10 @@ extern void SuspendClock(void);
 extern char *GetExtension(char *filename);
 extern void StrCp(char *dest, const char *src);
 extern int  BuildUserFileEntry(FileEntry *fe_ptr, 
-            int max_filename_len, int max_linkname_len, 
+            int filename_width, int linkname_width, 
             char *template, int linelen, char *line);
-extern int  GetUserFileEntryLength(int max_filename_len, 
-				   int max_linkname_len, char *template);
+extern int  GetVisualUserFileEntryLength( int max_visual_filename_len, 
+                                          int max_visual_linkname_len, char *template);
 extern LONGLONG AtoLL(char* cptr);
 extern void DisplayTree(WINDOW *win, int start_entry_no, int hilight_no);
 extern void ReCreateWindows(void);
@@ -1099,6 +1099,9 @@ extern int  RefreshDirWindow(void);
 extern char *StrLeft(const char *str, size_t count);
 extern char *StrRight(const char *str, size_t count);
 extern int  StrVisualLength(const char *str);
+extern int  VisualPositionToBytePosition(const char *str, int visual_pos);
 extern int  WAttrAddStr(WINDOW *win, int attr, char *str);
 extern char *Strndup(const char *s, int len);
 extern char *CutName(char *dest, char *src, unsigned int max_len);
+extern int  StrCaseCmp(char *s1, char *s2);
+extern int  StrNCaseCmp(char *s1, char *s2, unsigned int n);
